#include <stdio.h>

#include "b25.h"

int b25_open(ARIB_STD_B25 **b25, B_CAS_CARD **bcas, int32_t round, int strip, int emm)
{
	*b25 = create_arib_std_b25();
	if (*b25 == NULL) {
		fprintf(stderr, "failed to create arib_std_b25.\n");
		return -1;
	}
	if ((*b25)->set_multi2_round(*b25, round) < 0) {
		fprintf(stderr, "failed to b25->set_multi2_round.\n");
		return -1;
	}
	if ((*b25)->set_strip(*b25, strip ? 1 : 0) < 0) {
		fprintf(stderr, "failed to b25->set_strip.\n");
		return -1;
	}
	if ((*b25)->set_emm_proc(*b25, emm ? 1 : 0) < 0) {
		fprintf(stderr, "failed to b25->set_emm_proc.\n");
		return -1;
	}

	*bcas = create_b_cas_card();
	if (*bcas == NULL) {
		fprintf(stderr, "failed to create b_cas_card.\n");
		return -1;
	}
	if ((*bcas)->init(*bcas) < 0) {
		fprintf(stderr, "failed to bcas->init.\n");
		return -1;
	}

	if ((*b25)->set_b_cas_card(*b25, *bcas) < 0) {
		fprintf(stderr, "failed to b25->set_b_cas_card.\n");
		return -1;
	}
	
	return 0;
}

int b25_close(ARIB_STD_B25 **b25, B_CAS_CARD **bcas)
{
	if (*b25) {
		(*b25)->release(*b25);
		*b25 = NULL;
	}
	if (*bcas) {
		(*bcas)->release(*bcas);
		*bcas = NULL;
	}

	return 0;
}

int b25_put(ARIB_STD_B25 *b25, const uint8_t *buf, int32_t len)
{
	ARIB_STD_B25_BUFFER sbuf = { (uint8_t *)buf, len };
	if (b25->put(b25, &sbuf) < 0) {
		fprintf(stderr, "failed to b25->put.\n");
		return -1;
	}
	return 0;
} 

int b25_get(ARIB_STD_B25 *b25, const uint8_t **bufp, int32_t *len)
{
	ARIB_STD_B25_BUFFER dbuf = { NULL, 0 };
	if (b25->get(b25, &dbuf) < 0) {
		fprintf(stderr, "failed to b25->put.\n");
		*bufp = NULL;
		*len = 0;
		return -1;
	}
	*bufp = dbuf.data;
	*len = dbuf.size;
	return 0;
}

int b25_flush(ARIB_STD_B25 *b25)
{
	if (b25->flush(b25) < 0) {
		fprintf(stderr, "failed to b25->flush.\n");
		return -1;
	}
	return 0;
}
